package polys;

 /**
 CSE391 demo. Uses Java3D to view a list of triangles, to show
 how dynamically generated object can be controlled and displayed.
 Adapted from Sun's GearTest example, by GWH, Sept 2003, but object
 is a simple torus with structure determined by GUI controls.
 */

import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.universe.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import java.util.Vector;
import java.io.*;
import javax.swing.*;

public class J3Dviewer extends Applet {

    private SimpleUniverse u = null;
    private ctrlPanel ctrls = null;     // control panel
    private BranchGroup objSplice;      // where our thing gets spliced into scene
    private BranchGroup objBranch;      // group containing our thing
    private Appearance app;             // its appearance
    private Vector triangles;           // boundary of object
    public  static Vector triangles1;          // save level 1 for later use
    private JFileChooser stlFileChooser = new JFileChooser();
    public final int maxPoly = 11;       // max # poly families (1-based)
    public int nPoly = 0;                // # poly families
    public ctrlPoly[] poly = new ctrlPoly [maxPoly];  // array of controls

    // The following allows this to be run as an application
    // as well as an applet
    public static void main(String[] args) {
        if (args.length > 0) {
            System.out.println("Usage: java J3Dviewer");
            System.exit(0);
        }
        new MainFrame(new J3Dviewer(), 500, 500);
    }


    // Set up the two frames and their contents
    public void init() {
        // create control panel in separate JFrame
        ctrls = new ctrlPanel(this);
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        ctrls.setLocation(510, 0);
        ctrls.setSize(300,200);
        ctrls.setVisible(true);

        // put 3D canvas in this frame
        setLayout(new BorderLayout());
        GraphicsConfiguration config =
            SimpleUniverse.getPreferredConfiguration();
        Canvas3D c = new Canvas3D(config);
        add("Center", c);

        // Create a simple scene and attach it to the virtual universe
        BranchGroup scene = createSceneGraph();
        u = new SimpleUniverse(c);

        // This will move the ViewPlatform back a bit so the
        // objects in the scene can be viewed.
        u.getViewingPlatform().setNominalViewingTransform();

        u.addBranchGraph(scene);
    }


    // Create our scene graph, with lights, rotation, and object
    public BranchGroup createSceneGraph() {
	// Create the root of the branch graph
	BranchGroup objRoot = new BranchGroup();

        // Create a Transformgroup to scale all objects so they
        // appear in the scene.
        TransformGroup objScale = new TransformGroup();
        Transform3D t3d = new Transform3D();
        t3d.setScale(0.10);           // scale: larger value = larger appearance
        objScale.setTransform(t3d);
        objRoot.addChild(objScale);

	// Create a bounds for the background and lights
	BoundingSphere bounds =
	    new BoundingSphere(new Point3d(0.0,0.0,0.0), 100.0);

	// Set up the background
	Color3f bgColor = new Color3f(0.95f, 0.9f, 0.4f);
	Background bgNode = new Background(bgColor);
	bgNode.setApplicationBounds(bounds);
	objScale.addChild(bgNode);

	// Set up the global lights
	Color3f light1Color = new Color3f(1.0f, 1.0f, 0.9f);
	Vector3f light1Direction  = new Vector3f(4.0f, -7.0f, -12.0f);
	Color3f light2Color = new Color3f(0.3f, 0.3f, 0.4f);
	Vector3f light2Direction  = new Vector3f(-6.0f, -2.0f, -1.0f);
	Color3f ambientColor = new Color3f(0.1f, 0.1f, 0.1f);

	AmbientLight ambientLightNode = new AmbientLight(ambientColor);
	ambientLightNode.setInfluencingBounds(bounds);
	objScale.addChild(ambientLightNode);

	DirectionalLight light1
	    = new DirectionalLight(light1Color, light1Direction);
	light1.setInfluencingBounds(bounds);
	objScale.addChild(light1);

	DirectionalLight light2 = new DirectionalLight(light2Color, light2Direction);
	light2.setInfluencingBounds(bounds);
	objScale.addChild(light2);

	// Create the transform group node and initialize it to the
	// identity.  Enable the TRANSFORM_WRITE capability so that
	// our behavior code can modify it at runtime.  Add it to the
	// root of the subgraph.
	TransformGroup objTrans = new TransformGroup();
	objTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
	objScale.addChild(objTrans);

	// Create an Appearance.
	app = new Appearance();
	Color3f objColor = new Color3f(0.5f, 0.5f, 0.9f);
	Color3f black = new Color3f(0.0f, 0.0f, 0.0f);
	Color3f white = new Color3f(1.0f, 1.0f, 1.0f);
	app.setMaterial(new Material(objColor, black, objColor, white, 100.0f));

        // create thing to display first as list of triangles
        triangles = makeTetrahedron();                    // default object to start 
        Shape3D thing = new makeJ3DObj(triangles, app);   // then convert to J3D object
        objSplice = new BranchGroup();
        //Note Java3D bug: You can't OR these capabilities.  Set individually:
        objSplice.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
        objSplice.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
        objBranch = new BranchGroup();
        objBranch.setCapability(BranchGroup.ALLOW_DETACH);
	objTrans.addChild(objSplice);
        objSplice.addChild(objBranch);
        objBranch.addChild(thing);

	// Create a new Behavior object that will rotate the object and
	// add it into the scene graph.
	Transform3D yAxis = new Transform3D();
	Alpha rotationAlpha = new Alpha(-1, Alpha.INCREASING_ENABLE,
					0, 0, 60000, 0, 0, 0, 0, 0);
	RotationInterpolator rotator =
	    new RotationInterpolator(rotationAlpha, objTrans, yAxis,
				     0.0f, (float) Math.PI*2.0f);
	rotator.setSchedulingBounds(bounds);
	objTrans.addChild(rotator);

        // Have Java 3D perform optimizations on this scene graph.
        //objRoot.compile();        //crashes. (J3D bug?)
          
	return objRoot;
    }

    public void makeNewPoly(){             // create new poly control panel
        if (nPoly+1 >= maxPoly) return;      // ...or do nothing if too many
        nPoly = nPoly +1;
        ctrlPoly p = new ctrlPoly(this, nPoly);
        poly[nPoly] = p;
        p.setLocation(510 + (nPoly-1)*55, 210 + (nPoly-1)*55);        
    }
    
    // When controls change, regenerate the structure
    public void updateGeometry(){
      if (nPoly==0) triangles = makeTetrahedron();
      else {
          triangles = new Vector();              // combine all families 
          for (int i=1; i<=nPoly; i++) triangles.addAll(poly[i].triVec);
      }
      Shape3D newThing = new makeJ3DObj(triangles, app);   // convert format
      BranchGroup newBG = new BranchGroup();       // make a branchGroup for it
      newBG.addChild(newThing);
      //newBG.compile();                 //(crashes.  J3D bug?)
      objBranch.detach();                // remove old one
      objBranch=newBG;
      objBranch.setCapability(BranchGroup.ALLOW_DETACH);
      objSplice.addChild(objBranch);            // splice in new one
    }

    // when all done...
    public void destroy() {
	u.cleanup();
    }

    //------------------------------------------------------------------------- 
    public void writeFile(){                    // Write binary format STL file
        ctrls.statusshow("writing file...");  System.out.println("writing");
        Vector tris = triangles;                // data to output
        if (stlFileChooser.showSaveDialog(null) != JFileChooser.APPROVE_OPTION )
            return;
        String stlFileName = stlFileChooser.getSelectedFile().getPath();
        stlFileName = forceExtension(stlFileName, "stl");   // annex .stl if user didn't
        FileOutputStream stlStream;
        String headerText = "George W. Hart -- Orderly Tangles Generator";
        byte[] headerBytes = new byte[headerText.length()];
        for (int i=0; i<headerText.length(); i++)
            headerBytes[i] = (byte) headerText.charAt(i);
        
        try{
            File stlFile = new File(stlFileName);
            stlStream = new FileOutputStream(stlFile);
        }
        catch(FileNotFoundException ex){
            System.out.println("Error opening file " + stlFileName);
            return;
        }
        try{
            stlStream.write(headerBytes);                 // STL file header
            for (int i=headerText.length(); i<80; i++)
                stlStream.write((byte) 0);                  // pad to 80 chars
            int numTriangles = tris.size();
            stlStream.write((numTriangles>> 0) & 255);      // send out int as 4 bytes
            stlStream.write((numTriangles>> 8) & 255);
            stlStream.write((numTriangles>>16) & 255);
            stlStream.write((numTriangles>>24) & 255);
            for (int i=0; i<tris.size(); i++) {
                tri tun = (tri) tris.elementAt(i);          // unscaled triangle
                tri t= tun.scale(1.0);                      // scale
                xyz normal = t.normal();
                binaryWrite((float) normal.x, stlStream);
                binaryWrite((float) normal.y, stlStream);
                binaryWrite((float) normal.z, stlStream);
                binaryWrite((float) t.v1.x, stlStream);
                binaryWrite((float) t.v1.y, stlStream);
                binaryWrite((float) t.v1.z, stlStream);
                binaryWrite((float) t.v2.x, stlStream); 
                binaryWrite((float) t.v2.y, stlStream);
                binaryWrite((float) t.v2.z, stlStream);
                binaryWrite((float) t.v3.x, stlStream);
                binaryWrite((float) t.v3.y, stlStream);
                binaryWrite((float) t.v3.z, stlStream);
                stlStream.write((byte) 0);               // pad to 50 chars
                stlStream.write((byte) 0);               // pad to 50 chars
            }
            stlStream.close();
            ctrls.statusshow("file written");
        }
        catch(IOException ex){
            System.out.println("Error writing STL file " + stlFileName);
            return;
        }
        
    }
    
  // send out float as 4 bytes for STL binary format
  void binaryWrite (float x, FileOutputStream stream) throws IOException{
    int fourBytes = Float.floatToIntBits(x);
    stream.write((fourBytes>> 0) & 255);
    stream.write((fourBytes>> 8) & 255);
    stream.write((fourBytes>>16) & 255);
    stream.write((fourBytes>>24) & 255);
  }

  // annex given lower-case 3-letter extension to file name if user didn't
  public static String forceExtension(String name, String ext){
    String end = ("xxxx" + name).substring(name.length());
    if (end.toLowerCase().equals("." + ext)) return name;
    else return name + "." + ext;
  }
    
    
  //----------------------------------------------------------------------------
    // The following was used for testing the viewer:
    // create tetrahedron, as test object
    // as array of k triangles in form xyz[k][3]
    // where each xyz is a vertex
    //
    private Vector makeTetrahedron(){
      double scale=0.001;         // make it very small for this program  
        
      xyz v0=xyz.scale(scale, new xyz( 1, 1, 1));   // alternate vertices of cube
      xyz v1=xyz.scale(scale, new xyz( 1,-1,-1));
      xyz v2=xyz.scale(scale, new xyz(-1,-1, 1));
      xyz v3=xyz.scale(scale, new xyz(-1, 1,-1));

      Vector triVec = new Vector();
      triVec.add(new tri(v0,v2,v1));   // 4 triangles
      triVec.add(new tri(v0,v1,v3));
      triVec.add(new tri(v2,v0,v3));
      triVec.add(new tri(v2,v3,v1));
      
      return triVec;
    }

}
