package polys;

   /**
   Given list of triangles (each a triple of xyz's) create Java3D object.
   A Shape3D object is created containing one array of triangles.
   The appearance parameter is just passed through.
   Demo for CSE391 course, G. Hart, Sept 2003
   */

import java.lang.Math.*;
import java.util.Vector;
import javax.media.j3d.*;
import javax.vecmath.*;

public class makeJ3DObj extends javax.media.j3d.Shape3D{
  public makeJ3DObj(Vector triVec, Appearance app) {

    if (triVec.size() > 37000) 
        System.out.println("Skipping display of " + triVec.size() + " triangles");
    int nShow = triVec.size();             // max out display
    if (nShow>37000) nShow=500;
     
    TriangleArray tris = new TriangleArray(3*nShow,
                          GeometryArray.COORDINATES | GeometryArray.NORMALS);

    for (int i=0; i<nShow; i++){           // for each given triangle
      // float alpha = 1.0f;                  // scale factor
      tri t = (tri) triVec.elementAt(i);
      // t = t.scale(alpha);       
      xyz norm = t.normal();      // compute outward normal for lighting calculations

      // convert points and normals to vecmath format
      tris.setCoordinate(3*i+0, new Point3f(t.v1.x, t.v1.y, t.v1.z));
      tris.setNormal    (3*i+0, new Vector3f(norm.x, norm.y, norm.z));
      tris.setCoordinate(3*i+1, new Point3f(t.v2.x, t.v2.y, t.v2.z));
      tris.setNormal    (3*i+1, new Vector3f(norm.x, norm.y, norm.z));
      tris.setCoordinate(3*i+2, new Point3f(t.v3.x, t.v3.y, t.v3.z));
      tris.setNormal    (3*i+2, new Vector3f(norm.x, norm.y, norm.z));
    }

    this.setGeometry(tris);              // put triangles in this object
    this.setAppearance(app);             // pass along given appearance
    //this.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);  // allow rotation
  }

}